/*
	quick-launch control-panel applet
	
	@author	Daniele Maio (b0unc3) 
	@date	18/07/2010
	@update	12/08/2010
	@ver	0.6
	
*/
#include <hildon-cp-plugin/hildon-cp-plugin-interface.h>
#include <gtk/gtk.h>
#include <hildon/hildon.h>
#include <glib.h>
#include <string.h>
#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <gconf/gconf-client.h>


#include "quick-launch.h"

static gboolean
filter_func                                     (GtkTreeModel *model,
                                                 GtkTreeIter  *iter,
                                                 gchar        *text,
                                                 gpointer      data)
{
    gboolean retvalue;
    gchar *rowtext = NULL;

    gtk_tree_model_get (model, iter, 0, &rowtext, -1);
    retvalue = g_str_has_prefix (rowtext, text);
    g_free (rowtext);

    return retvalue;
}

void
button_toggled ( GtkWidget *wdg, gpointer user_data )
{
	gtk_widget_set_sensitive ( GTK_WIDGET(confirm), TRUE );
}

gint
sort_iter_compare_func (GtkTreeModel *model,
                        GtkTreeIter  *a,
                        GtkTreeIter  *b,
                        gpointer      userdata)
{
	gchar *name1, *name2;
	gint ret = 0;



        gtk_tree_model_get(model, a, 0, &name1, -1);
        gtk_tree_model_get(model, b, 0, &name2, -1);

        if (name1 == NULL || name2 == NULL)
        {
          if (name1 == NULL && name2 == NULL)
              ret = 0;

          ret = (name1 == NULL) ? -1 : 1;
        }
        else
        {
          ret = g_utf8_collate(name1,name2);
        }

        g_free(name1);
        g_free(name2);

        return ret;
}

gchar *
g_strreplace (gchar * string, gchar * search, gchar * replace)
{
        gchar **sp;
        gchar *ret;

        sp = g_strsplit(string,search,-1);
        ret =  (g_strjoinv (replace, sp)); //g_strsplit (string, search, -1)));

        g_strfreev(sp);

        return ret;
}


void
save_info(GtkWidget *wid, gpointer user_data)
{
	GtkIconTheme	*it;
        GtkIconInfo	*info;
        GConfClient* client;
        int i;
        
        it = gtk_icon_theme_new();
       	client = GCONF_CLIENT(user_data);
	GSList  *lst = NULL;
	GError  *error;

        for(i=0;i<4;i++) {
		        
		if ( names[i] != NULL ) lst = g_slist_append (lst, names[i]);
		if ( icons[i] != NULL ) {
		info = gtk_icon_theme_lookup_icon (it, g_strdup_printf(icons[i]), 64, 0);
		if (info)
			icons[i] = g_strdup (gtk_icon_info_get_filename (info));
		
		lst = g_slist_append (lst, icons[i]);
		}
		if ( ossos[i] != NULL ) lst = g_slist_append (lst, ossos[i]);
		if ( bins[i] != NULL ) lst = g_slist_append (lst, bins[i]);        

		if ( lst != NULL ) gconf_client_set_list ( client, g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i]), GCONF_VALUE_STRING, lst, &error);
		lst = NULL;
	}
	
	//	GtkWidget 	*shutter,*pkey;
	if ( hildon_check_button_get_active ( HILDON_CHECK_BUTTON(shutter) ) ) 
		gconf_client_set_bool (client, g_strdup_printf("%sshutter",APP_GCONF_KEY), TRUE, NULL);
//		gconf_client_set_string(client, g_strdup_printf("%sshutter",APP_GCONF_KEY), "active", NULL);
	else
		gconf_client_set_bool (client, g_strdup_printf("%sshutter",APP_GCONF_KEY), FALSE, NULL);
		
		
	if ( hildon_check_button_get_active ( HILDON_CHECK_BUTTON(pkey) ) )
		gconf_client_set_bool (client, g_strdup_printf("%spower_key",APP_GCONF_KEY), TRUE, NULL);
//		gconf_client_set_string(client, g_strdup_printf("%spower_key",APP_GCONF_KEY), "active", NULL);
	else
		gconf_client_set_bool (client, g_strdup_printf("%spower_key",APP_GCONF_KEY), FALSE, NULL);
//		gconf_client_set_string(client, g_strdup_printf("%spower_key",APP_GCONF_KEY), "inactive", NULL);

	
	gtk_widget_set_sensitive ( GTK_WIDGET(confirm), FALSE );
	
	gtk_widget_destroy ( GTK_WIDGET(window) );

}

void
parse_settings(GConfClient* client)
{
	GError *err;
	GtkWidget *icoim;
	int i;
	GSList *lst = NULL;
	gpointer *nm,*in,*o,*b;
	GdkPixbuf *pixim;
	
	for(i=0;i<4;i++) {
		if (gconf_entry_get_value (g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i])) != NULL) {
			lst = gconf_client_get_list (GCONF_CLIENT(client),g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[i]), GCONF_VALUE_STRING, &err);
			if ( lst != NULL ) {
				nm = g_slist_nth_data (lst, 0);
				in = g_slist_nth_data (lst, 1);
				o =  g_slist_nth_data (lst, 2);
				b =  g_slist_nth_data (lst, 3);
				names[i] = (gchar *)nm;
				icons[i] = (gchar *)in;
				ossos[i] = (gchar *)o;
				bins[i] = (gchar *)b;
				if ( g_strcmp0(in,"") != 0 ) {
					pixim = gdk_pixbuf_new_from_file ( (gchar *)in, NULL );
					icoim = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple ( pixim, 48,48, GDK_INTERP_BILINEAR ));
				
					hildon_button_set_image ( HILDON_BUTTON(btns[i]), icoim );
					hildon_button_set_text ( HILDON_BUTTON(btns[i]), g_strreplace((gchar *)nm," ","\n")  , "Tap to set");
				}
			}
		}
	}
	
	if (gconf_entry_get_value(g_strdup_printf("%sshutter",APP_GCONF_KEY)) != NULL)	
		if ( gconf_client_get_bool (GCONF_CLIENT(client), g_strdup_printf("%sshutter",APP_GCONF_KEY), &err) )
			hildon_check_button_set_active ( HILDON_CHECK_BUTTON(shutter), TRUE );
		else hildon_check_button_set_active ( HILDON_CHECK_BUTTON(shutter), FALSE );
		
	if (gconf_entry_get_value(g_strdup_printf("%spower_key",APP_GCONF_KEY)) != NULL)	
		if ( gconf_client_get_bool (GCONF_CLIENT(client), g_strdup_printf("%spower_key",APP_GCONF_KEY), &err) )
			hildon_check_button_set_active ( HILDON_CHECK_BUTTON(pkey), TRUE );
		else hildon_check_button_set_active ( HILDON_CHECK_BUTTON(pkey), FALSE );
	
}

gboolean
free_slot ( GtkWidget *btn, gpointer data )
{
	gint pos = (gint)data;
	if ( gconf_client_unset (GCONF_CLIENT(client), g_strdup_printf("%s%s",APP_GCONF_KEY,group_name[(gint)pos]), NULL ) ) {
		names[pos] = "";
		icons[pos] = "";
		ossos[pos] = "";
		bins[pos] = "";
		hildon_button_set_image ( HILDON_BUTTON(btns[(gint)pos]), NULL );
		hildon_button_set_text ( HILDON_BUTTON(btns[(gint)pos]), "Tap to set", "" );
		hildon_banner_show_information(NULL, NULL, "Slot freed");
		gtk_widget_hide ( GTK_WIDGET(gtk_widget_get_toplevel(btn)) );
	} else hildon_banner_show_information(NULL,NULL, "Slot not freed");
	
	return FALSE;
}
gboolean
set_shortcut ( HildonTouchSelector * selector, gint column, gpointer user_data )
{

	//load icon from selected application
	gchar *current_selection = NULL;
	gint	pos;
 
   	current_selection = hildon_touch_selector_get_current_text (HILDON_TOUCH_SELECTOR(selector));
   	GDir		*d;
   	gchar 		*ico;
   	GtkIconTheme	*it;
   	GtkWidget 	*icoim;
   	GKeyFile 	*dsk;
   	GError		*error;

	pos = (gint)user_data;
	
	
	if ( current_selection != NULL ) {
		it = gtk_icon_theme_new();
		d = g_dir_open(APPS_DIR,0,NULL);
		if ( d ) {
			setlocale( LC_ALL, "" );
		        const gchar *file;
		        while ( (file=g_dir_read_name(d)) ) {
		                if ( g_strrstr(file,".desktop") != NULL ) {
		                        /* Add item to the column */
	//      
		                        
					dsk = g_key_file_new();
					gchar *n,*oss,*bin;
					n = NULL;
				
					if ( g_key_file_load_from_file ( dsk, g_strdup_printf("%s%s",APPS_DIR,file), G_KEY_FILE_KEEP_COMMENTS, &error ) ){
						if ( g_key_file_has_key ( dsk, "Desktop Entry", "X-Text-Domain", &error ) ) {
						//get name from gettext

							textdomain( g_key_file_get_value( dsk, "Desktop Entry", "X-Text-Domain", &error ) );
							n = gettext( g_key_file_get_value ( dsk, "Desktop Entry", "Name", &error ) );
						} else if ( g_key_file_has_key ( dsk, "Desktop Entry", "Name", &error ) ) {
							//get name from entry
							n = g_key_file_get_value ( dsk, "Desktop Entry", "Name", &error );
						}
					
						if ( g_key_file_has_key ( dsk, "Desktop Entry", "X-Osso-Service", &error ) ) 
							oss = g_key_file_get_value ( dsk, "Desktop Entry", "X-Osso-Service", &error );
						else oss = "";
					
						if ( g_key_file_has_key ( dsk, "Desktop Entry", "Exec", &error ) ) 
							bin = g_key_file_get_value ( dsk, "Desktop Entry", "Exec", &error );
						else bin = "";
					
				                if ( n != NULL && g_strrstr(n,"_") == NULL && g_ascii_strncasecmp (n,current_selection,-1) == 0 ) {
				                	//get icon
				                	if ( g_key_file_has_key ( dsk, "Desktop Entry", "Icon", &error ) ) {
				                		ico = g_key_file_get_value ( dsk, "Desktop Entry", "Icon", &error );
				                		icoim = gtk_image_new_from_icon_name ( ico, GTK_ICON_SIZE_LARGE_TOOLBAR );
								hildon_button_set_image ( HILDON_BUTTON(btns[(gint)pos]), icoim );
								hildon_button_set_text ( HILDON_BUTTON(btns[(gint)pos]), g_strreplace(n," ","\n"), "Tap to set" );
								names[pos] = n;
								icons[pos] = ico;
								ossos[pos] = oss;
								bins[pos] = bin;
								gtk_widget_set_sensitive ( GTK_WIDGET(confirm), TRUE );
				                	}
						}
					}
				
					g_key_file_free (dsk);
		                }
		        }
		}
		g_dir_close(d);
        }
        
        return FALSE;
}


void 
add_apps( HildonTouchSelector *btn, gpointer p)
{
	//load all available applications
	GDir	*d;
	GKeyFile *dsk;
	GError *error;
	
	d = g_dir_open(APPS_DIR,0,NULL);
	if ( d ) {
		setlocale( LC_ALL, "" );
                const gchar *file;
                while ( (file=g_dir_read_name(d)) ) {
                        if ( g_strrstr(file,".desktop") != NULL ) {
                                /* Add item to the column */
//      
                                
				dsk = g_key_file_new();
				gchar *n;
				n = NULL;
				
				if ( g_key_file_load_from_file ( dsk, g_strdup_printf("%s%s",APPS_DIR,file), G_KEY_FILE_KEEP_COMMENTS, &error ) ){
					if ( g_key_file_has_key ( dsk, "Desktop Entry", "X-Text-Domain", &error ) ) {
					//getname from gettext

						textdomain( g_key_file_get_value( dsk, "Desktop Entry", "X-Text-Domain", &error ) );
						n = gettext( g_key_file_get_value ( dsk, "Desktop Entry", "Name", &error ) );
					} else if ( g_key_file_has_key ( dsk, "Desktop Entry", "Name", &error ) ) {
						n = g_key_file_get_value ( dsk, "Desktop Entry", "Name", &error );
					}
					
		                        if ( n != NULL && g_strrstr(n,"_") == NULL ) hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (btn), n); 
				}
				
				g_key_file_free (dsk);
                        }
                }
        }
        g_dir_close(d);
        

         
	hildon_touch_selector_set_live_search ( HILDON_TOUCH_SELECTOR(btn), TRUE );
        
       /* 
 	GtkTreeModel *m;
 	GtkTreeSortable *sortable;
 	       
        m = hildon_touch_selector_get_model (HILDON_TOUCH_SELECTOR(btn),0);
        sortable = GTK_TREE_SORTABLE(m);
	gtk_tree_sortable_set_sort_func(sortable, NULL, sort_iter_compare_func, NULL,NULL );
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sortable), 0, GTK_SORT_ASCENDING);
	hildon_touch_selector_set_model ( HILDON_TOUCH_SELECTOR(btn), 0, GTK_TREE_MODEL(sortable) );*/
	GtkWidget *tmp;
	GtkWidget *b;
	
	tmp = gtk_widget_get_toplevel (btn);
	if ( GTK_IS_WIDGET (tmp)) {
		b = hildon_button_new_with_text (HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL, "Clear Slot", "Click to free this slot");

		gtk_box_pack_start ( GTK_BOX(tmp), b, FALSE,TRUE, 1);
		g_signal_connect ( G_OBJECT(b), "clicked", G_CALLBACK(free_slot), (gpointer)p ); 
		gtk_widget_show_all ( GTK_WIDGET(tmp) );
	}
	
	GtkTreeModelFilter *filter;
	
	filter = GTK_TREE_MODEL_FILTER (gtk_tree_view_get_model (GTK_TREE_VIEW (btn)));
	hildon_live_search_set_filter (btn, filter);
	hildon_live_search_set_visible_func (btn, filter_func, NULL, NULL);	


}

osso_return_t execute(osso_context_t *osso, gpointer data, gboolean user_activated)
{
	HildonProgram   *program;
	GtkWidget 	*main_box;
	GtkWidget	*conf_box;
	GtkWidget	*action_box;

	GtkWidget	*sel1,*sel2,*sel3,*sel4;
	
	program = HILDON_PROGRAM(hildon_program_get_instance());
	g_set_application_name("Quick Launch - Configuration");

	window = HILDON_WINDOW(hildon_window_new());
	hildon_program_add_window(program, HILDON_WINDOW(window));

	client = gconf_client_get_default();
	
	main_box = gtk_vbox_new ( FALSE,3 );
	conf_box = gtk_hbox_new ( FALSE,0 );
	action_box = gtk_vbox_new ( FALSE,0 );
	
	btns[0] = hildon_picker_button_new(HILDON_SIZE_AUTO_WIDTH, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
	btns[1] = hildon_picker_button_new(HILDON_SIZE_AUTO_WIDTH, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
	btns[2] = hildon_picker_button_new(HILDON_SIZE_AUTO_WIDTH, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
	btns[3] = hildon_picker_button_new(HILDON_SIZE_AUTO_WIDTH, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
	hildon_button_set_text ( HILDON_BUTTON(btns[0]), "Tap to set", "" );
	hildon_button_set_text ( HILDON_BUTTON(btns[1]), "Tap to set", "" );
	hildon_button_set_text ( HILDON_BUTTON(btns[2]), "Tap to set", "" );
	hildon_button_set_text ( HILDON_BUTTON(btns[3]), "Tap to set", "" );
	gtk_widget_set_size_request ( GTK_WIDGET(btns[0]), 200,200 );
	gtk_widget_set_size_request ( GTK_WIDGET(btns[1]), 200,200 );
	gtk_widget_set_size_request ( GTK_WIDGET(btns[2]), 200,200 );
	gtk_widget_set_size_request ( GTK_WIDGET(btns[3]), 200,200 );
	
	sel1 = hildon_touch_selector_new_text();
	sel2 = hildon_touch_selector_new_text();
	sel3 = hildon_touch_selector_new_text();
	sel4 = hildon_touch_selector_new_text();
	add_apps(HILDON_TOUCH_SELECTOR(sel1),(gpointer)0);
	add_apps(HILDON_TOUCH_SELECTOR(sel2),(gpointer)1);
	add_apps(HILDON_TOUCH_SELECTOR(sel3),(gpointer)2);
	add_apps(HILDON_TOUCH_SELECTOR(sel4),(gpointer)3);	
	
	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (btns[0]), HILDON_TOUCH_SELECTOR (sel1));
	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (btns[1]), HILDON_TOUCH_SELECTOR (sel2));
	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (btns[2]), HILDON_TOUCH_SELECTOR (sel3));
	hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (btns[3]), HILDON_TOUCH_SELECTOR (sel4));
	
	gtk_box_pack_start(GTK_BOX(conf_box), btns[0], FALSE, TRUE, 1);
	gtk_box_pack_start(GTK_BOX(conf_box), btns[1], FALSE, TRUE, 1);
	gtk_box_pack_start(GTK_BOX(conf_box), btns[2], FALSE, TRUE, 1);
	gtk_box_pack_start(GTK_BOX(conf_box), btns[3], FALSE, TRUE, 1);

	confirm = hildon_button_new(HILDON_SIZE_AUTO, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
	hildon_button_set_text ( HILDON_BUTTON(confirm), "Ok", NULL );
	hildon_button_set_image ( HILDON_BUTTON(confirm), gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON) );
	
	shutter = hildon_check_button_new ( HILDON_SIZE_FINGER_HEIGHT );
	pkey = hildon_check_button_new ( HILDON_SIZE_FINGER_HEIGHT );
	gtk_button_set_label (GTK_BUTTON (shutter), "Enable launch on shutter open");
	gtk_button_set_label (GTK_BUTTON (pkey), "Enable launch on power key press");

    	g_signal_connect (shutter, "toggled", G_CALLBACK (button_toggled), NULL);
	g_signal_connect (pkey, "toggled", G_CALLBACK (button_toggled), NULL);

	
	gtk_box_pack_start ( GTK_BOX(action_box), shutter, FALSE,FALSE,0 );
	gtk_box_pack_start ( GTK_BOX(action_box), pkey, FALSE,FALSE,0 );
	gtk_box_pack_start ( GTK_BOX(action_box), confirm, FALSE,FALSE,0 );

	gtk_box_pack_start ( GTK_BOX(main_box), conf_box, TRUE, TRUE, 1 );
	gtk_box_pack_start ( GTK_BOX(main_box), action_box, TRUE, FALSE, 1 );

	gtk_container_add ( GTK_CONTAINER(window), main_box );


	gconf_client_add_dir(client,
                       g_strdup_printf("%s%s",GCONF_DIR,APP_NAME),
                       GCONF_CLIENT_PRELOAD_NONE,
                       NULL);
                       
	g_signal_connect ( G_OBJECT(confirm), "clicked", G_CALLBACK(save_info), client );
	
	gtk_widget_set_sensitive ( GTK_WIDGET(confirm), FALSE );
	
	gtk_widget_show_all ( GTK_WIDGET(window) );
	
	parse_settings(client);

	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR(sel1), 0, -1);
        hildon_touch_selector_unselect_all (HILDON_TOUCH_SELECTOR(sel1), 0);
	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR(sel2), 0, -1);
        hildon_touch_selector_unselect_all (HILDON_TOUCH_SELECTOR(sel2), 0);
	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR(sel3), 0, -1);
        hildon_touch_selector_unselect_all (HILDON_TOUCH_SELECTOR(sel3), 0);
	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR(sel4), 0, -1);
        hildon_touch_selector_unselect_all (HILDON_TOUCH_SELECTOR(sel4), 0);

	g_signal_connect ( G_OBJECT(sel1), "changed", G_CALLBACK(set_shortcut), (gpointer)0 );
	g_signal_connect ( G_OBJECT(sel2), "changed", G_CALLBACK(set_shortcut), (gpointer)1 );	
	g_signal_connect ( G_OBJECT(sel3), "changed", G_CALLBACK(set_shortcut), (gpointer)2 );
	g_signal_connect ( G_OBJECT(sel4), "changed", G_CALLBACK(set_shortcut), (gpointer)3 );
	
	
	return OSSO_OK;
}

osso_return_t save_state(osso_context_t *osso, gpointer data)
{
	/* ... save state ... */

	return OSSO_OK;
}

